@@ -2,11 +2,23 @@
 Common classes related to LLVM.
 """
 
-__version__ = '0.9'
+__version__ = '0.10.0'
 
 
 from weakref import WeakValueDictionary
+import _core
 
+#===----------------------------------------------------------------------===
+# LLVM Version
+#===----------------------------------------------------------------------===
+
+version = _core.LLVMGetVersion()
+
+def require_version_at_least(major, minor):
+    '''Sentry to guard version requirement
+    '''
+    if version < (major, minor):
+        raise Exception(major, minor)
 
 #===----------------------------------------------------------------------===
 # Exceptions
