@@ -88,12 +88,12 @@ template <> PyObject* pycap_new<LLVMByteOrdering>(LLVMByteOrdering i)
     return PyLong_FromLong(static_cast<unsigned long>(i));
 }
 
-template <> PyObject* pycap_new<unsigned long long>(unsigned long long ull)
+template <> PyObject* pycap_new<unsigned PY_LONG_LONG>(unsigned PY_LONG_LONG ull)
 {
     return PyLong_FromUnsignedLongLong(ull);
 }
 
-template <> PyObject* pycap_new<long long>(long long ll)
+template <> PyObject* pycap_new<PY_LONG_LONG>(PY_LONG_LONG ll)
 {
     return PyLong_FromLongLong(ll);
 }
@@ -226,12 +226,12 @@ _wLLVMGetBitcodeFromModule(PyObject *self, PyObject *args)
     if (!m ) return NULL;
 
     size_t len;
-    unsigned const char *ubytes = LLVMGetBitcodeFromModule(m, &len) ;
+    const unsigned char *ubytes = LLVMGetBitcodeFromModule(m, &len) ;
     if ( !ubytes ) Py_RETURN_NONE;
 
     const char *chars = reinterpret_cast<const char*>(ubytes) ;
     PyObject *ret = PyBytes_FromStringAndSize(chars, len);
-    LLVMDisposeMessage(const_cast<char*>(chars));
+    delete [] ubytes;
     return ret;
     LLVMPY_CATCH_ALL
 }
@@ -259,7 +259,7 @@ _wLLVMGetNativeCodeFromModule(PyObject * self, PyObject * args)
     const char *chars = reinterpret_cast<const char*>(ubytes) ;
     PyObject * ret = PyBytes_FromStringAndSize(chars, len);
 
-    LLVMDisposeMessage(const_cast<char*>(chars));
+    delete [] ubytes;
 
     return ret;
     LLVMPY_CATCH_ALL
@@ -278,9 +278,9 @@ _wLLVMLinkModules(PyObject *self, PyObject *args)
     const LLVMModuleRef dest = pycap_get<LLVMModuleRef>(dest_obj ) ;
     const LLVMModuleRef src = pycap_get<LLVMModuleRef>(src_obj) ;
 
-    PyObject *ret;
-    char *errmsg;
-    if (!LLVMLinkModules(dest, src, (LLVMLinkerMode)mode, &errmsg)) {
+    PyObject *ret = NULL;
+    char *errmsg = NULL;
+    if (LLVMLinkModules(dest, src, (LLVMLinkerMode)mode, &errmsg)) {
         if (errmsg) {
             ret = PyUnicode_FromString(errmsg);
             LLVMDisposeMessage(errmsg);
@@ -483,7 +483,7 @@ _wLLVMConstInt(PyObject *self, PyObject *args)
 {
     LLVMPY_TRY
     PyObject *obj;
-    unsigned long long n;
+    unsigned PY_LONG_LONG n;
     int sign_extend;
 
     if (!PyArg_ParseTuple(args, "OKi", &obj, &n, &sign_extend))
@@ -761,6 +761,7 @@ _wrap_obj2obj(LLVMInstGetOpcode,      LLVMValueRef, int)
 _wrap_obj2str(LLVMInstGetOpcodeName,  LLVMValueRef)
 
 _wrap_objstrobj2none(LLVMInstSetMetaData, LLVMValueRef, LLVMValueRef)
+_wrap_obj2none(LLVMInstructionEraseFromParent,    LLVMValueRef)
 
 /*===-- Call Sites (Call or Invoke) --------------------------------------===*/
 
@@ -877,6 +878,9 @@ _wrap_objobj2obj(LLVMBuildFree, LLVMBuilderRef, LLVMValueRef, LLVMValueRef)
 _wrap_objobjstr2obj(LLVMBuildLoad, LLVMBuilderRef, LLVMValueRef, LLVMValueRef)
 _wrap_objobjobj2obj(LLVMBuildStore, LLVMBuilderRef, LLVMValueRef, LLVMValueRef, LLVMValueRef)
 _wrap_objobjliststr2obj(LLVMBuildGEP, LLVMBuilderRef, LLVMValueRef, LLVMValueRef, LLVMValueRef)
+_wrap_objobjliststr2obj(LLVMBuildInBoundsGEP, LLVMBuilderRef, LLVMValueRef, LLVMValueRef, LLVMValueRef)
+
+
 
 _wrap_objint2none(LLVMLdSetAlignment, LLVMValueRef)
 _wrap_objint2none(LLVMStSetAlignment, LLVMValueRef)
@@ -987,6 +991,11 @@ _wrap_obj2obj(LLVMPassManagerBuilderGetSizeLevel, LLVMPassManagerBuilderRef, int
 _wrap_objint2none(LLVMPassManagerBuilderSetVectorize, LLVMPassManagerBuilderRef)
 _wrap_obj2obj(LLVMPassManagerBuilderGetVectorize, LLVMPassManagerBuilderRef, int)
 
+#if LLVM_VERSION_MAJOR >= 3 && LLVM_VERSION_MINOR >= 2
+_wrap_objint2none(LLVMPassManagerBuilderSetLoopVectorize, LLVMPassManagerBuilderRef)
+_wrap_obj2obj(LLVMPassManagerBuilderGetLoopVectorize, LLVMPassManagerBuilderRef, int)
+#endif //llvm-3.2
+
 _wrap_objint2none(LLVMPassManagerBuilderSetDisableUnitAtATime,
                   LLVMPassManagerBuilderRef)
 _wrap_obj2obj(LLVMPassManagerBuilderGetDisableUnitAtATime,
@@ -1030,6 +1039,7 @@ _wrap_none2str(LLVMDumpPasses)
 _wrap_objstr2obj(LLVMAddPassByName, LLVMPassManagerRef, int)
 
 
+_wrap_objobj2none(LLVMAddPass, LLVMPassManagerRef, LLVMPassRef)
 _wrap_none2none(LLVMInitializePasses)
 
 _wrap_none2obj(LLVMInitializeNativeTarget, int)
@@ -1051,96 +1061,17 @@ _wrap_none2none(LLVMInitializePTXAsmPrinter)
 /*===----------------------------------------------------------------------===*/
 /* Passes                                                                     */
 /*===----------------------------------------------------------------------===*/
-/*
 
-#define _wrap_pass(P)   \
-_wrap_obj2none( LLVMAdd ## P ## Pass, LLVMPassManagerRef)
+_wrap_str2obj(LLVMCreatePassByName, LLVMPassRef)
+_wrap_obj2none(LLVMDisposePass, LLVMPassRef)
+_wrap_obj2str(LLVMGetPassName, LLVMPassRef)
+_wrap_obj2none(LLVMPassDump, LLVMPassRef)
 
-_wrap_pass( AAEval )
-_wrap_pass( AggressiveDCE )
-_wrap_pass( AliasAnalysisCounter )
-_wrap_pass( AlwaysInliner )
-_wrap_pass( ArgumentPromotion )
-_wrap_pass( BasicAliasAnalysis )
-_wrap_pass( BlockPlacement )
-_wrap_pass( BreakCriticalEdges )
-_wrap_pass( CFGSimplification )
-_wrap_pass( CodeGenPrepare )
-_wrap_pass( ConstantMerge )
-_wrap_pass( ConstantPropagation )
-_wrap_pass( DbgInfoPrinter )
-_wrap_pass( DeadArgElimination )
-_wrap_pass( DeadCodeElimination )
-_wrap_pass( DeadInstElimination )
-_wrap_pass( DeadStoreElimination )
-_wrap_pass( DemoteRegisterToMemory )
-_wrap_pass( DomOnlyPrinter )
-_wrap_pass( DomOnlyViewer )
-_wrap_pass( DomPrinter )
-_wrap_pass( DomViewer )
-_wrap_pass( EdgeProfiler )
-_wrap_pass( FunctionAttrs )
-_wrap_pass( FunctionInlining )
-//_wrap_pass( GEPSplitter )
-_wrap_pass( GlobalDCE )
-_wrap_pass( GlobalOptimizer )
-_wrap_pass( GlobalsModRef )
-_wrap_pass( GVN )
-_wrap_pass( IndVarSimplify )
-_wrap_pass( InstCount )
-_wrap_pass( InstructionCombining )
-_wrap_pass( InstructionNamer )
-_wrap_pass( IPConstantPropagation )
-_wrap_pass( IPSCCP )
-_wrap_pass( JumpThreading )
-_wrap_pass( LazyValueInfo )
-_wrap_pass( LCSSA )
-_wrap_pass( LICM )
-//_wrap_pass( LiveValues )
-_wrap_pass( LoopDeletion )
-_wrap_pass( LoopDependenceAnalysis )
-_wrap_pass( LoopExtractor )
-//_wrap_pass( LoopIndexSplit )
-_wrap_pass( LoopRotate )
-_wrap_pass( LoopSimplify )
-_wrap_pass( LoopStrengthReduce )
-_wrap_pass( LoopUnroll )
-_wrap_pass( LoopUnswitch )
-_wrap_pass( LowerInvoke )
-_wrap_pass( LowerSwitch )
-_wrap_pass( MemCpyOpt )
-_wrap_pass( MergeFunctions )
-_wrap_pass( NoAA )
-_wrap_pass( NoProfileInfo )
-_wrap_pass( OptimalEdgeProfiler )
-_wrap_pass( PartialInlining )
-//_wrap_pass( PartialSpecialization )
-_wrap_pass( PostDomOnlyPrinter )
-_wrap_pass( PostDomOnlyViewer )
-_wrap_pass( PostDomPrinter )
-_wrap_pass( PostDomViewer )
-_wrap_pass( ProfileEstimator )
-_wrap_pass( ProfileLoader )
-_wrap_pass( ProfileVerifier )
-_wrap_pass( PromoteMemoryToRegister )
-_wrap_pass( PruneEH )
-_wrap_pass( Reassociate )
-_wrap_pass( ScalarEvolutionAliasAnalysis )
-_wrap_pass( ScalarReplAggregates )
-_wrap_pass( SCCP )
-//_wrap_pass( SimplifyHalfPowrLibCalls )
-_wrap_pass( SimplifyLibCalls )
-_wrap_pass( SingleLoopExtractor )
-_wrap_pass( StripDeadPrototypes )
-_wrap_pass( StripNonDebugSymbols )
-_wrap_pass( StripSymbols )
-//_wrap_pass( StructRetPromotion )
-_wrap_pass( TailCallElimination )
-//_wrap_pass( TailDuplication )
-_wrap_pass( UnifyFunctionExitNodes )
+_wrap_str2obj(LLVMCreateTargetLibraryInfo, LLVMPassRef)
 
-_wrap_pass( Internalize2 )
-*/
+#if LLVM_VERSION_MAJOR >= 3 && LLVM_VERSION_MINOR >= 2
+_wrap_obj2obj(LLVMCreateTargetTransformInfo, LLVMTargetMachineRef, LLVMPassRef)
+#endif
 
 
 /*===----------------------------------------------------------------------===*/
@@ -1157,17 +1088,43 @@ static PyObject *
 _wLLVMTargetMachineLookup(PyObject * self, PyObject * args)
 {
     LLVMPY_TRY
-    const char *arch;
+    const char *triple;
     const char *cpu;
     const char *features;
-    int opt;
+    int opt, codemodel;
 
-    if (!PyArg_ParseTuple(args, "sssi", &arch, &cpu, &features, &opt))
+    if (!PyArg_ParseTuple(args, "sssii", &triple, &cpu, &features,
+                          &opt, &codemodel))
         return NULL;
 
     std::string error;
-    LLVMTargetMachineRef tm = LLVMTargetMachineLookup(arch, cpu, features, opt,
-                                                      error);
+    LLVMTargetMachineRef tm = LLVMTargetMachineLookup(triple, cpu, features,
+                                                      opt, codemodel, error);
+    if(!error.empty()){
+        PyErr_SetString(PyExc_RuntimeError, error.c_str());
+        return NULL;
+    }
+    return pycap_new<LLVMTargetMachineRef>(tm);
+    LLVMPY_CATCH_ALL
+}
+
+
+static PyObject *
+_wLLVMCreateTargetMachine(PyObject * self, PyObject * args)
+{
+    LLVMPY_TRY
+    const char *triple;
+    const char *cpu;
+    const char *features;
+    int opt, codemodel;
+
+    if (!PyArg_ParseTuple(args, "sssii", &triple, &cpu, &features,
+                                         &opt, &codemodel))
+        return NULL;
+
+    std::string error;
+    LLVMTargetMachineRef tm = LLVMCreateTargetMachine(triple, cpu, features,
+                                                      opt, codemodel, error);
     if(!error.empty()){
         PyErr_SetString(PyExc_RuntimeError, error.c_str());
         return NULL;
@@ -1292,41 +1249,68 @@ _wLLVMEngineBuilderCreate(PyObject *self, PyObject *args)
     LLVMPY_CATCH_ALL
 }
 
+static PyObject *
+_wLLVMEngineBuilderCreateTM(PyObject *self, PyObject *args)
+{
+    LLVMPY_TRY
+    PyObject *obj, *tm;
+    if (!PyArg_ParseTuple(args, "OO", &obj, &tm))
+        return NULL;
+
+    const LLVMEngineBuilderRef objref = pycap_get<LLVMEngineBuilderRef>( obj );
+    const LLVMTargetMachineRef tmref = pycap_get<LLVMTargetMachineRef>( tm );
+    std::string outmsg;
+
+    const LLVMExecutionEngineRef ee = LLVMEngineBuilderCreateTM(objref, tmref,
+                                                                outmsg);
+
+    PyObject * ret;
+    if( !ee ){ // check if error message is set.
+        ret = PyUnicode_FromString(outmsg.c_str());
+    }else{
+        ret = pycap_new<LLVMExecutionEngineRef>(ee);
+    }
+    
+    return ret;
+    LLVMPY_CATCH_ALL
+}
+
 /*===----------------------------------------------------------------------===*/
 /* Execution Engine                                                           */
 /*===----------------------------------------------------------------------===*/
 
-static PyObject *
-_wLLVMCreateExecutionEngine(PyObject *self, PyObject *args)
-{
-    LLVMPY_TRY
-    PyObject *obj;
-    int force_interpreter;
-    char *outmsg = 0;
-    int error;
-
-    if (!PyArg_ParseTuple(args, "Oi", &obj, &force_interpreter))
-        return NULL;
-
-    const LLVMModuleRef mod = pycap_get<LLVMModuleRef>( obj ) ;
-
-    LLVMExecutionEngineRef ee;
-    if (force_interpreter)
-        error = LLVMCreateInterpreterForModule(&ee, mod, &outmsg);
-    else
-        error = LLVMCreateJITCompilerForModule(&ee, mod, 1 /*fast*/, &outmsg);
-
-    PyObject *ret;
-    if (error) {
-        ret = PyUnicode_FromString(outmsg);
-        LLVMDisposeMessage(outmsg);
-    } else {
-        ret = pycap_new<LLVMExecutionEngineRef>(ee);
-    }
-
-    return ret;
-    LLVMPY_CATCH_ALL
-}
+// Use EngineBuilder
+//static PyObject *
+//_wLLVMCreateExecutionEngine(PyObject *self, PyObject *args)
+//{
+//    LLVMPY_TRY
+//    PyObject *obj;
+//    int force_interpreter;
+//    char *outmsg = 0;
+//    int error;
+//
+//    if (!PyArg_ParseTuple(args, "Oi", &obj, &force_interpreter))
+//        return NULL;
+//
+//    const LLVMModuleRef mod = pycap_get<LLVMModuleRef>( obj ) ;
+//
+//    LLVMExecutionEngineRef ee;
+//    if (force_interpreter)
+//        error = LLVMCreateInterpreterForModule(&ee, mod, &outmsg);
+//    else
+//        error = LLVMCreateJITCompilerForModule(&ee, mod, 1 /*fast*/, &outmsg);
+//
+//    PyObject *ret;
+//    if (error) {
+//        ret = PyUnicode_FromString(outmsg);
+//        LLVMDisposeMessage(outmsg);
+//    } else {
+//        ret = pycap_new<LLVMExecutionEngineRef>(ee);
+//    }
+//
+//    return ret;
+//    LLVMPY_CATCH_ALL
+//}
 
 static PyObject *
 _wLLVMGetPointerToFunction(PyObject *self, PyObject *args)
@@ -1345,6 +1329,48 @@ _wLLVMGetPointerToFunction(PyObject *self, PyObject *args)
     LLVMPY_CATCH_ALL
 }
 
+static PyObject *
+_wLLVMGetPointerToGlobal(PyObject *self, PyObject *args)
+{
+    LLVMPY_TRY
+    PyObject *obj_ee;
+    PyObject *obj_val;
+
+    if (!PyArg_ParseTuple(args, "OO", &obj_ee, &obj_val))
+        return NULL;
+
+    const LLVMExecutionEngineRef ee = pycap_get<LLVMExecutionEngineRef>( obj_ee ) ;
+    const LLVMValueRef val = pycap_get<LLVMValueRef>( obj_val ) ;
+
+    return PyLong_FromVoidPtr(LLVMGetPointerToGlobal(ee, val));
+    LLVMPY_CATCH_ALL
+}
+
+static PyObject *
+_wLLVMAddGlobalMapping(PyObject *self, PyObject *args)
+{
+    LLVMPY_TRY
+    PyObject *obj_ee;
+    PyObject *obj_val;
+    PyObject *obj_long;
+    void* address;
+
+    if (!PyArg_ParseTuple(args, "OOO", &obj_ee, &obj_val, &obj_long))
+        return NULL;
+
+    address = PyLong_AsVoidPtr(obj_long);
+    if (PyErr_Occurred()) return NULL;
+
+    const LLVMExecutionEngineRef ee = pycap_get<LLVMExecutionEngineRef>( obj_ee );
+    const LLVMValueRef val = pycap_get<LLVMValueRef>( obj_val );
+
+    LLVMAddGlobalMapping(ee, val, address);
+
+    Py_RETURN_NONE;
+
+    LLVMPY_CATCH_ALL
+}
+
 /* the args should have been ptr, num */
 LLVMGenericValueRef LLVMRunFunction2(LLVMExecutionEngineRef EE,
     LLVMValueRef F, LLVMGenericValueRef *Args, unsigned NumArgs)
@@ -1384,6 +1410,8 @@ _wLLVMRemoveModule2(PyObject *self, PyObject *args)
 }
 
 _wrap_obj2none(LLVMDisposeExecutionEngine, LLVMExecutionEngineRef)
+_wrap_objint2none(LLVMExecutionEngineDisableLazyCompilation,
+                  LLVMExecutionEngineRef)
 _wrap_objobjlist2obj(LLVMRunFunction2, LLVMExecutionEngineRef,
     LLVMValueRef, LLVMGenericValueRef, LLVMGenericValueRef)
 _wrap_obj2obj(LLVMGetExecutionEngineTargetData, LLVMExecutionEngineRef,
@@ -1405,11 +1433,11 @@ _wLLVMCreateGenericValueOfInt(PyObject *self, PyObject *args)
 {
     LLVMPY_TRY
     PyObject *obj1;
-    unsigned long long n;
+    unsigned PY_LONG_LONG n;
     int is_signed;
     LLVMGenericValueRef gv;
 
-    if (!PyArg_ParseTuple(args, "OLi", &obj1, &n, &is_signed))
+    if (!PyArg_ParseTuple(args, "OKi", &obj1, &n, &is_signed))
         return NULL;
 
     const LLVMTypeRef ty = pycap_get<LLVMTypeRef>( obj1 ) ;
@@ -1443,10 +1471,10 @@ _wLLVMCreateGenericValueOfPointer(PyObject *self, PyObject *args)
     LLVMPY_TRY
 //    PyObject *obj1;
 //    LLVMTypeRef ty; //unused?
-    unsigned long long n_;
+    unsigned PY_LONG_LONG n_;
     size_t n;
 
-    if (!PyArg_ParseTuple(args, "L", &n_))
+    if (!PyArg_ParseTuple(args, "K", &n_))
         return NULL;
 
     n=n_;
@@ -1462,7 +1490,7 @@ _wLLVMGenericValueToInt(PyObject *self, PyObject *args)
     LLVMPY_TRY
     PyObject *obj1;
     int is_signed;
-    unsigned long long val;
+    unsigned PY_LONG_LONG val;
 
     if (!PyArg_ParseTuple(args, "Oi", &obj1, &is_signed))
         return NULL;
@@ -1471,7 +1499,7 @@ _wLLVMGenericValueToInt(PyObject *self, PyObject *args)
 
     val = LLVMGenericValueToInt(gv, is_signed);
     return is_signed ?
-        PyLong_FromLongLong((long long)val) :
+        PyLong_FromLongLong((PY_LONG_LONG)val) :
         PyLong_FromUnsignedLongLong(val);
     LLVMPY_CATCH_ALL
 }
@@ -1516,6 +1544,14 @@ _wrap_obj2none(LLVMDisposeGenericValue, LLVMGenericValueRef)
 /* Misc                                                                       */
 /*===----------------------------------------------------------------------===*/
 
+static PyObject *
+_wLLVMGetVersion(PyObject *self, PyObject *args)
+{
+    int major = LLVM_VERSION_MAJOR;
+    int minor = LLVM_VERSION_MINOR;
+    return Py_BuildValue("(i,i)", major, minor);
+}
+
 _wrap_objintlist2obj(LLVMGetIntrinsic, LLVMModuleRef, LLVMTypeRef,
     LLVMValueRef)
 
@@ -1562,6 +1598,9 @@ _wLLVMParseEnvOpts(PyObject *self, PyObject *args)
 
 _wrap_obj2obj(LLVMInlineFunction, LLVMValueRef, int)
 
+_wrap_none2str(LLVMDefaultTargetTriple)
+
+
 /* Expose the void* inside a PyCObject as a PyLong. This allows us to
  * use it as a unique ID. */
 static PyObject *
@@ -1857,6 +1896,7 @@ static PyMethodDef core_methods[] = {
     _method( LLVMInstGetOpcodeName )
 
     _method( LLVMInstSetMetaData )
+    _method( LLVMInstructionEraseFromParent )
 
     /* Call Sites (Call or Invoke) */
     _method( LLVMSetInstructionCallConv )
@@ -1929,6 +1969,7 @@ static PyMethodDef core_methods[] = {
     _method( LLVMBuildLoad )
     _method( LLVMBuildStore )
     _method( LLVMBuildGEP )
+    _method( LLVMBuildInBoundsGEP )
 
     _method( LLVMLdSetAlignment )
     _method( LLVMStSetAlignment )
@@ -1987,7 +2028,10 @@ static PyMethodDef core_methods[] = {
 
     _method( LLVMPassManagerBuilderSetVectorize )
     _method( LLVMPassManagerBuilderGetVectorize )
-
+#if LLVM_VERSION_MAJOR >= 3 && LLVM_VERSION_MINOR >= 2
+    _method( LLVMPassManagerBuilderSetLoopVectorize )
+    _method( LLVMPassManagerBuilderGetLoopVectorize )
+#endif // llvm-3.2
 
     _method( LLVMPassManagerBuilderSetDisableUnitAtATime )
     _method( LLVMPassManagerBuilderGetDisableUnitAtATime )
@@ -2013,6 +2057,7 @@ static PyMethodDef core_methods[] = {
     _method( LLVMDisposePassManager )
     _method( LLVMDumpPasses )
     _method( LLVMAddPassByName )
+    _method( LLVMAddPass )
     _method( LLVMInitializePasses )
 
     _method( LLVMInitializeNativeTarget )
@@ -2030,98 +2075,11 @@ static PyMethodDef core_methods[] = {
     _method( LLVMInitializePTXAsmPrinter )
 # endif
 #endif
-    /* Passes */
-
-    /*
-    _pass( AAEval )
-    _pass( AggressiveDCE )
-    _pass( AliasAnalysisCounter )
-    _pass( AlwaysInliner )
-    _pass( ArgumentPromotion )
-    _pass( BasicAliasAnalysis )
-    _pass( BlockPlacement )
-    _pass( BreakCriticalEdges )
-    _pass( CFGSimplification )
-    _pass( CodeGenPrepare )
-    _pass( ConstantMerge )
-    _pass( ConstantPropagation )
-    _pass( DbgInfoPrinter )
-    _pass( DeadArgElimination )
-    _pass( DeadCodeElimination )
-    _pass( DeadInstElimination )
-    _pass( DeadStoreElimination )
-    _pass( DemoteRegisterToMemory )
-    _pass( DomOnlyPrinter )
-    _pass( DomOnlyViewer )
-    _pass( DomPrinter )
-    _pass( DomViewer )
-    _pass( EdgeProfiler )
-    _pass( FunctionAttrs )
-    _pass( FunctionInlining )
-    //_pass( GEPSplitter )
-    _pass( GlobalDCE )
-    _pass( GlobalOptimizer )
-    _pass( GlobalsModRef )
-    _pass( GVN )
-    _pass( IndVarSimplify )
-    _pass( InstCount )
-    _pass( InstructionCombining )
-    _pass( InstructionNamer )
-    _pass( IPConstantPropagation )
-    _pass( IPSCCP )
-    _pass( JumpThreading )
-    _pass( LazyValueInfo )
-    _pass( LCSSA )
-    _pass( LICM )
-    //_pass( LiveValues )
-    _pass( LoopDeletion )
-    _pass( LoopDependenceAnalysis )
-    _pass( LoopExtractor )
-    //_pass( LoopIndexSplit )
-    _pass( LoopRotate )
-    _pass( LoopSimplify )
-    _pass( LoopStrengthReduce )
-    _pass( LoopUnroll )
-    _pass( LoopUnswitch )
-    _pass( LowerInvoke )
-    _pass( LowerSwitch )
-    _pass( MemCpyOpt )
-    _pass( MergeFunctions )
-    _pass( NoAA )
-    _pass( NoProfileInfo )
-    _pass( OptimalEdgeProfiler )
-    _pass( PartialInlining )
-    //_pass( PartialSpecialization )
-    _pass( PostDomOnlyPrinter )
-    _pass( PostDomOnlyViewer )
-    _pass( PostDomPrinter )
-    _pass( PostDomViewer )
-    _pass( ProfileEstimator )
-    _pass( ProfileLoader )
-    _pass( ProfileVerifier )
-    _pass( PromoteMemoryToRegister )
-    _pass( PruneEH )
-    _pass( Reassociate )
-    _pass( ScalarEvolutionAliasAnalysis )
-    _pass( ScalarReplAggregates )
-    _pass( SCCP )
-    //_pass( SimplifyHalfPowrLibCalls )
-    _pass( SimplifyLibCalls )
-    _pass( SingleLoopExtractor )
-    _pass( StripDeadPrototypes )
-    _pass( StripNonDebugSymbols )
-    _pass( StripSymbols )
-    //_pass( StructRetPromotion )
-    _pass( TailCallElimination )
-    //_pass( TailDuplication )
-    _pass( UnifyFunctionExitNodes )
-
-    _pass( Internalize2 )
-    */
 
     /* Target Machine */
     _method( LLVMTargetMachineFromEngineBuilder )
     _method( LLVMDisposeTargetMachine )
+    _method( LLVMCreateTargetMachine )
     _method( LLVMTargetMachineLookup )
     _method( LLVMTargetMachineEmitFile )
     _method( LLVMTargetMachineGetTargetData )
@@ -2162,12 +2120,17 @@ static PyMethodDef core_methods[] = {
     _method( LLVMEngineBuilderSetMCPU )
     _method( LLVMEngineBuilderSetMAttrs )
     _method( LLVMEngineBuilderCreate )
+    _method( LLVMEngineBuilderCreateTM )
 
     /* Execution Engine */
-    _method( LLVMCreateExecutionEngine )
+// Use EngineBuilder instead
+//    _method( LLVMCreateExecutionEngine )
     _method( LLVMDisposeExecutionEngine )
+    _method( LLVMExecutionEngineDisableLazyCompilation )
     _method( LLVMRunFunction2 )
     _method( LLVMGetPointerToFunction )
+    _method( LLVMGetPointerToGlobal )
+    _method( LLVMAddGlobalMapping )
     _method( LLVMGetExecutionEngineTargetData )
     _method( LLVMRunStaticConstructors )
     _method( LLVMRunStaticDestructors )
@@ -2184,11 +2147,24 @@ static PyMethodDef core_methods[] = {
     _method( LLVMGenericValueToPointer )
     _method( LLVMDisposeGenericValue )
 
+    /* Passes */
+    _method( LLVMCreatePassByName )
+    _method( LLVMDisposePass )
+    _method( LLVMGetPassName )
+    _method( LLVMPassDump )
+
+    _method( LLVMCreateTargetLibraryInfo )
+
+#if LLVM_VERSION_MAJOR >= 3 && LLVM_VERSION_MINOR >= 2
+    _method( LLVMCreateTargetTransformInfo )
+#endif
+
     /* Misc */
+    _method( LLVMGetVersion )
     _method( LLVMGetIntrinsic )
     _method( LLVMLoadLibraryPermanently )
     _method( LLVMParseEnvOpts )
-
+    _method( LLVMDefaultTargetTriple )
     _method( LLVMInlineFunction )
     _method( PyCObjectVoidPtrToPyLong )
     { NULL }
