@@ -289,6 +289,7 @@ def check_is_type_struct(obj):  _util.check_gen(obj, StructType)
 def check_is_value(obj):        _util.check_gen(obj, Value)
 def check_is_constant(obj):     _util.check_gen(obj, Constant)
 def check_is_function(obj):     _util.check_gen(obj, Function)
+def check_is_global_value(obj): _util.check_gen(obj, GlobalValue)
 def check_is_basic_block(obj):  _util.check_gen(obj, BasicBlock)
 def check_is_module(obj):       _util.check_gen(obj, Module)
 
@@ -450,7 +451,7 @@ class Module(llvm.Ownable, llvm.Cacheable):
         llvm::Module::AnyPointerSize."""
         return _core.LLVMModuleGetPointerSize(self.ptr)
 
-    def link_in(self, other):
+    def link_in(self, other, preserve=False):
         """Link the `other' module into this one.
 
         The `other' module is linked into this one such that types,
@@ -463,12 +464,16 @@ class Module(llvm.Ownable, llvm.Cacheable):
         Linker class.
         """
         check_is_module(other)
-        other.forget() # remove it from object cache
-        ret = _core.LLVMLinkModules(self.ptr, other.ptr)
-        if isinstance(ret, str):
-            raise llvm.LLVMException(ret)
-        # Do not try to destroy the other module's llvm::Module*.
-        other._own(llvm.DummyOwner())
+        if not preserve:
+            other.forget() # remove it from object cache
+        result = _core.LLVMLinkModules(self.ptr, other.ptr, int(bool(preserve)))
+        if result is not None:
+            raise llvm.LLVMException(result)
+        if not preserve:
+            # Prevent user from using the other module
+            other.ptr = None
+            # Use a dummy owner to prevent other.ptr to be deleted
+            other._own(llvm.DummyOwner())
 
     def get_type_named(self, name):
         """Return a Type object with the given name."""
@@ -517,6 +522,8 @@ class Module(llvm.Ownable, llvm.Cacheable):
         Checks module for errors. Raises `llvm.LLVMException' on any
         error."""
         ret = _core.LLVMVerifyModule(self.ptr)
+        # Note: LLVM has a bug in preverifier that will always abort
+        #       the process upon failure.
         if ret != "":
             raise llvm.LLVMException(ret)
 
@@ -1503,6 +1510,9 @@ class Function(GlobalValue):
     def verify(self):
         # Although we're just asking LLVM to return the success or
         # failure, it appears to print result to stderr and abort.
+
+        # Note: LLVM has a bug in preverifier that will always abort
+        #       the process upon failure.
         return _core.LLVMVerifyFunction(self.ptr) != 0
 
 #===----------------------------------------------------------------------===
@@ -1654,6 +1664,8 @@ class Instruction(User):
     def opcode_name(self):
         return _core.LLVMInstGetOpcodeName(self.ptr)
 
+    def erase_from_parent(self):
+        return _core.LLVMInstructionEraseFromParent(self.ptr)
 
 class CallOrInvokeInstruction(Instruction):
 
@@ -1862,6 +1874,9 @@ class Builder(object):
 
     def branch(self, bblk):
         check_is_basic_block(bblk)
+        if __debug__:
+            for instr in self.basic_block.instructions:
+                assert not instr.is_terminator, "BasicBlock can only have one terminator"
         return _make_value(_core.LLVMBuildBr(self.ptr, bblk.ptr))
 
     def cbranch(self, if_value, then_blk, else_blk):
@@ -2051,11 +2066,14 @@ class Builder(object):
             inst.set_volatile(volatile)
         return inst
 
-    def gep(self, ptr, indices, name=""):
+    def gep(self, ptr, indices, name="", inbounds=False):
         check_is_value(ptr)
         index_ptrs = unpack_values(indices)
-        return _make_value(
-            _core.LLVMBuildGEP(self.ptr, ptr.ptr, index_ptrs, name))
+        if inbounds:
+            ret = _core.LLVMBuildInBoundsGEP(self.ptr, ptr.ptr, index_ptrs, name)
+        else:
+            ret = _core.LLVMBuildGEP(self.ptr, ptr.ptr, index_ptrs, name)
+        return _make_value(ret)
 
     # casts and extensions
 
