@@ -53,7 +53,6 @@
 #include "llvm/Support/MemoryBuffer.h"
 #include "llvm/Support/CallSite.h"
 #include "llvm/Support/FormattedStream.h"
-#include "llvm/Target/TargetData.h"
 #include "llvm/Support/TargetRegistry.h"
 #include "llvm/Support/TargetSelect.h"
 #include "llvm/Support/Host.h"
@@ -75,8 +74,16 @@
 #include "llvm/Linker.h"
 #include "llvm/Support/SourceMgr.h"
 #include <llvm/InlineAsm.h>
+#include <llvm/Support/PassNameParser.h>
+#include <llvm/Target/TargetLibraryInfo.h>
 
 
+#if LLVM_VERSION_MAJOR >= 3 && LLVM_VERSION_MINOR >= 2
+    #include "llvm/DataLayout.h"
+#else
+    #include "llvm/Target/TargetData.h"
+#endif
+
 // LLVM-C includes
 #include "llvm-c/Core.h"
 #include "llvm-c/ExecutionEngine.h"
@@ -89,6 +96,7 @@ namespace llvm{
 DEFINE_SIMPLE_CONVERSION_FUNCTIONS(EngineBuilder, LLVMEngineBuilderRef)
 DEFINE_SIMPLE_CONVERSION_FUNCTIONS(TargetMachine, LLVMTargetMachineRef)
 DEFINE_SIMPLE_CONVERSION_FUNCTIONS(NamedMDNode, LLVMNamedMDRef)
+DEFINE_SIMPLE_CONVERSION_FUNCTIONS(Pass, LLVMPassRef)
 
 template<typename T>
 inline T **unwrap(LLVMTypeRef *Tys, unsigned Length) {
@@ -105,7 +113,7 @@ class PassRegistryPrinter : public llvm::PassRegistrationListener{
 public:
     std::ostringstream stringstream;
 
-    void passEnumerate(const llvm::PassInfo * pass_info){
+    inline virtual void passEnumerate(const llvm::PassInfo * pass_info){
         stringstream << pass_info->getPassArgument()
                      << "\t"
                      << pass_info->getPassName()
@@ -134,8 +142,68 @@ const CodeGenOpt::Level OptLevelMap[] = {
     CodeGenOpt::Default,
     CodeGenOpt::Aggressive,
 };
+
+const CodeModel::Model CodeModelMap[] = {
+    CodeModel::Default,
+    CodeModel::JITDefault,
+    CodeModel::Small,
+    CodeModel::Kernel,
+    CodeModel::Medium,
+    CodeModel::Large,
+};
+
+
 } // end anony namespace
 
+#if LLVM_VERSION_MAJOR >= 3 && LLVM_VERSION_MINOR >= 2
+LLVMPassRef LLVMCreateTargetTransformInfo(LLVMTargetMachineRef tmref){
+    using namespace llvm;
+    TargetMachine * tm = unwrap(tmref);
+    Pass * tti = new TargetTransformInfo(tm->getScalarTargetTransformInfo(),
+                                         tm->getVectorTargetTransformInfo());
+    return wrap(tti);
+}
+#endif
+
+LLVMPassRef LLVMCreateTargetLibraryInfo(const char * triple){
+    using namespace llvm;
+    Pass * tli = new TargetLibraryInfo(Triple(triple));
+    return wrap(tli);
+}
+
+const char * LLVMDefaultTargetTriple(){
+    return strdup(llvm::sys::getDefaultTargetTriple().c_str());
+}
+
+LLVMPassRef LLVMCreatePassByName(const char *name){
+    using namespace llvm;
+    const PassInfo * pi = Pass::lookupPassInfo(StringRef(name));
+    if (pi) {
+        return wrap(pi->createPass());
+    } else { // cannot find pass
+        return NULL;
+    }
+}
+
+void LLVMDisposePass(LLVMPassRef passref){
+    using namespace llvm;
+    delete unwrap(passref);
+}
+
+const char * LLVMGetPassName(LLVMPassRef passref){
+    using namespace llvm;
+    return unwrap(passref)->getPassName();
+}
+
+void LLVMPassDump(LLVMPassRef passref){
+    using namespace llvm;
+    return unwrap(passref)->dump();
+}
+
+void LLVMAddPass(LLVMPassManagerRef pmref, LLVMPassRef passref){
+    using namespace llvm;
+    unwrap(pmref)->add(unwrap(passref));
+}
 
 LLVMValueRef LLVMGetFunctionFromInlineAsm(LLVMTypeRef funcType,
                                           const char inlineAsm[],
@@ -273,6 +341,7 @@ int LLVMInitializeNativeTargetAsmPrinter()
 
 LLVMTargetMachineRef LLVMTargetMachineLookup(const char *arch, const char *cpu,
                                              const char *features, int opt,
+                                             int codemodel,
                                              std::string &error)
 {
     using namespace llvm;
@@ -309,11 +378,12 @@ LLVMTargetMachineRef LLVMTargetMachineLookup(const char *arch, const char *cpu,
     }
 
     TargetOptions no_target_options;
-    TargetMachine * tm = TheTarget->createTargetMachine(
-                                     TheTriple.str(), cpu, features,
-                                     no_target_options,
-                                     Reloc::Default, CodeModel::Default,
-                                     OptLevelMap[opt]);
+    TargetMachine * tm = TheTarget->createTargetMachine(TheTriple.str(), cpu,
+                                                        features,
+                                                        no_target_options,
+                                                        Reloc::Default,
+                                                        CodeModelMap[codemodel],
+                                                        OptLevelMap[opt]);
 
     if (!tm){
         error = "Cannot create target machine";
@@ -322,6 +392,31 @@ LLVMTargetMachineRef LLVMTargetMachineLookup(const char *arch, const char *cpu,
     return wrap(tm);
 }
 
+LLVMTargetMachineRef LLVMCreateTargetMachine(const char *triple,
+                                             const char *cpu,
+                                             const char *features,
+                                             int opt,
+                                             int codemodel,
+                                             std::string &error)
+{
+    using namespace llvm;
+
+    std::string TheTriple = triple;
+    const Target * TheTarget = TargetRegistry::lookupTarget(TheTriple, error);
+    if (!TheTarget) return NULL;
+
+    TargetOptions no_target_options;
+    TargetMachine * tm = TheTarget->createTargetMachine(TheTriple, cpu, features,
+                                                        no_target_options,
+                                                        Reloc::Default,
+                                                        CodeModelMap[codemodel],
+                                                        OptLevelMap[opt]);
+    if (!tm) {
+        error = "Cannot create target machine";
+        return NULL;
+    }
+    return wrap(tm);
+}
 
 LLVMTargetMachineRef LLVMTargetMachineFromEngineBuilder(LLVMEngineBuilderRef eb)
 {
@@ -355,13 +450,21 @@ unsigned char* LLVMTargetMachineEmitFile(LLVMTargetMachineRef tmref,
     TargetMachine * tm = unwrap(tmref);
 
     PassManager pm;
-
+#if LLVM_VERSION_MAJOR >= 3 && LLVM_VERSION_MINOR >= 2
+    if (!tm->getDataLayout()){
+        error = "No target data in target machine";
+        return NULL;
+    }
+    pm.add(new DataLayout(*tm->getDataLayout()));
+#else
     if (!tm->getTargetData()){
         error = "No target data in target machine";
         return NULL;
     }
-
     pm.add(new TargetData(*tm->getTargetData()));
+#endif
+
+
 
     bool failed;
     if( assembly ) {
@@ -427,7 +530,11 @@ void LLVMPrintRegisteredTargetsForVersion(){
 LLVMTargetDataRef LLVMTargetMachineGetTargetData(LLVMTargetMachineRef tm)
 {
     using namespace llvm;
+#if LLVM_VERSION_MAJOR >= 3 && LLVM_VERSION_MINOR >= 2
+    return wrap(new DataLayout(*unwrap(tm)->getDataLayout()));
+#else
     return wrap(new TargetData(*unwrap(tm)->getTargetData()));
+#endif
 }
 
 unsigned char* LLVMGetNativeCodeFromModule(LLVMModuleRef module, int assembly,
@@ -632,6 +739,22 @@ LLVMExecutionEngineRef LLVMEngineBuilderCreate(LLVMEngineBuilderRef eb, std::str
     }
 }
 
+LLVMExecutionEngineRef LLVMEngineBuilderCreateTM(LLVMEngineBuilderRef eb,
+                                                 LLVMTargetMachineRef tm,
+                                                 std::string & error)
+{
+    using namespace llvm;
+    LLVMExecutionEngineRef ret;
+
+    ret = wrap(unwrap(eb)->setErrorStr(&error).create(unwrap(tm)));
+
+    if ( !error.empty() ) { // error string is set
+        return NULL;
+    } else {
+        return ret;
+    }
+}
+
 int LLVMPassManagerBuilderGetOptLevel(LLVMPassManagerBuilderRef pmb)
 {
     return llvm::unwrap(pmb)->OptLevel;
@@ -651,6 +774,19 @@ int LLVMPassManagerBuilderGetVectorize(LLVMPassManagerBuilderRef pmb){
     return llvm::unwrap(pmb)->Vectorize;
 }
 
+#if LLVM_VERSION_MAJOR >= 3 && LLVM_VERSION_MINOR >= 2
+void LLVMPassManagerBuilderSetLoopVectorize(LLVMPassManagerBuilderRef pmb,
+                                            int flag)
+{
+    llvm::unwrap(pmb)->LoopVectorize = flag;
+}
+
+int LLVMPassManagerBuilderGetLoopVectorize(LLVMPassManagerBuilderRef pmb){
+    return llvm::unwrap(pmb)->LoopVectorize;
+}
+#endif // llvm-3.2
+
+
 int LLVMPassManagerBuilderGetDisableUnitAtATime(LLVMPassManagerBuilderRef pmb)
 {
     return llvm::unwrap(pmb)->DisableUnitAtATime;
@@ -992,7 +1128,8 @@ void LLVMSetDoesNotThrow(LLVMValueRef fn, int DoesNotThrow)
     llvm::Function *fnp = llvm::unwrap<llvm::Function>(fn);
     assert(fnp);
 
-    return fnp->setDoesNotThrow((bool)DoesNotThrow);
+    if ((bool)DoesNotThrow)
+    	fnp->setDoesNotThrow();
 }
 
 
@@ -1054,21 +1191,15 @@ LLVMModuleRef LLVMGetModuleFromBitcode(const char *bitcode, unsigned bclen,
 }
 
 #if LLVM_VERSION_MAJOR <= 3 && LLVM_VERSION_MINOR < 2
-unsigned LLVMLinkModules(LLVMModuleRef dest, LLVMModuleRef src, unsigned int mode,
-			 char **out)
-{
-    llvm::Module *sourcep = llvm::unwrap(src);
-    assert(sourcep);
-    llvm::Module *destinationp = llvm::unwrap(dest);
-    assert(destinationp);
-
-    std::string msg;
-    if (llvm::Linker::LinkModules(destinationp, sourcep, mode, &msg)) {
-        *out = strdup(msg.c_str());
-        return 0;
-    }
-
-    return 1;
+// Shamelessly copy from LLVM-3.2
+unsigned LLVMLinkModules(LLVMModuleRef Dest, LLVMModuleRef Src,
+                         int Mode, char **OutMessages) {
+    std::string Messages;
+    unsigned Result = Linker::LinkModules(unwrap(Dest), unwrap(Src), Mode,
+                                          OutMessages? &Messages : 0);
+    if (OutMessages)
+            *OutMessages = strdup(Messages.c_str());
+    return Result;
 }
 #endif
 
@@ -1117,6 +1248,11 @@ unsigned LLVMLoadLibraryPermanently(const char* filename, char **errmsg)
     return 1;
 }
 
+void LLVMExecutionEngineDisableLazyCompilation(LLVMExecutionEngineRef ee, int flag)
+{
+    llvm::unwrap(ee)->DisableLazyCompilation(flag);
+}
+
 void *LLVMGetPointerToFunction(LLVMExecutionEngineRef ee, LLVMValueRef fn)
 {
     llvm::ExecutionEngine *eep = llvm::unwrap(ee);
