@@ -10,6 +10,7 @@ import subprocess
 import tempfile
 
 is_py3k = bool(sys.version_info[0] == 3)
+BITS = tuple.__itemsize__ * 8
 
 if is_py3k:
     from io import StringIO
@@ -412,6 +413,23 @@ class TestEngineBuilder(unittest.TestCase):
 
         self.run_foo(ee, module)
 
+
+    def test_enginebuilder_with_tm(self):
+        tm = le.TargetMachine.new()
+        module = self.make_test_module()
+        ee = EngineBuilder.new(module).create(tm)
+
+        with self.assertRaises(llvm.LLVMException):
+            # Ensure the targetmachine is owned.
+            llvm._util.check_is_unowned(tm)
+        
+
+        with self.assertRaises(llvm.LLVMException):
+            # Ensure the module is owned.
+            llvm._util.check_is_unowned(module)
+
+        self.run_foo(ee, module)
+
     def test_enginebuilder_force_jit(self):
         module = self.make_test_module()
         ee = EngineBuilder.new(module).force_jit().create()
@@ -434,6 +452,46 @@ tests.append(TestEngineBuilder)
 
 # ---------------------------------------------------------------------------
 
+class TestExecutionEngine(unittest.TestCase):
+    def test_get_pointer_to_global(self):
+        module = lc.Module.new(str(self))
+        gvar = module.add_global_variable(Type.int(), 'hello')
+        X = 1234
+        gvar.initializer = lc.Constant.int(Type.int(), X)
+
+        ee = le.ExecutionEngine.new(module)
+        ptr = ee.get_pointer_to_global(gvar)
+        from ctypes import c_void_p, cast, c_int, POINTER
+        casted = cast(c_void_p(ptr), POINTER(c_int))
+        self.assertEqual(X, casted[0])
+
+    def test_add_global_mapping(self):
+        module = lc.Module.new(str(self))
+        gvar = module.add_global_variable(Type.int(), 'hello')
+
+        fnty = lc.Type.function(Type.int(), [])
+        foo = module.add_function(fnty, name='foo')
+        bldr = lc.Builder.new(foo.append_basic_block('entry'))
+        bldr.ret(bldr.load(gvar))
+
+        ee = le.ExecutionEngine.new(module)
+        from ctypes import c_int, addressof, CFUNCTYPE
+        value = 0xABCD
+        value_ctype = c_int(value)
+        value_pointer = addressof(value_ctype)
+
+        ee.add_global_mapping(gvar, value_pointer)
+
+        foo_addr = ee.get_pointer_to_function(foo)
+        prototype = CFUNCTYPE(c_int)
+        foo_callable = prototype(foo_addr)
+        self.assertEqual(foo_callable(), value)
+
+
+
+tests.append(TestExecutionEngine)
+# ---------------------------------------------------------------------------
+
 class TestObjCache(unittest.TestCase):
 
     def test_objcache(self):
@@ -618,6 +676,10 @@ class TestNative(unittest.TestCase):
         self.assertEqual(s, 0xab)
 
     def test_assembly(self):
+        if sys.platform == 'darwin':
+            # skip this test on MacOSX for now
+            return
+
         m = self._make_module()
         output = m.to_native_assembly()
 
@@ -628,6 +690,10 @@ class TestNative(unittest.TestCase):
         self._compile(src)
 
     def test_object(self):
+        if sys.platform == 'darwin':
+            # skip this test on MacOSX for now
+            return
+
         m = self._make_module()
         output = m.to_native_object()
 
@@ -802,6 +868,168 @@ class TestOpaque(unittest.TestCase):
 
 tests.append(TestOpaque)
 
+# ---------------------------------------------------------------------------
+class TestCPUSupport(unittest.TestCase):
+
+    def _build_test_module(self):
+        mod     = Module.new('test')
+
+        float   = Type.double()
+        mysinty = Type.function( float, [float] )
+        mysin   = mod.add_function(mysinty, "mysin")
+        block   = mysin.append_basic_block("entry")
+        b       = Builder.new(block)
+
+        sqrt = Function.intrinsic(mod, lc.INTR_SQRT, [float])
+        pow  = Function.intrinsic(mod, lc.INTR_POWI, [float])
+        cos  = Function.intrinsic(mod, lc.INTR_COS,  [float])
+
+        mysin.args[0].name = "x"
+        x    = mysin.args[0]
+        one  = Constant.real(float, "1")
+        cosx = b.call(cos, [x], "cosx")
+        cos2 = b.call(pow, [cosx, Constant.int(Type.int(), 2)], "cos2")
+        onemc2 = b.fsub(one, cos2, "onemc2") # Should use fsub
+        sin  = b.call(sqrt, [onemc2], "sin")
+        b.ret(sin)
+        return mod, mysin
+
+    def _template(self, mattrs):
+        mod, func = self._build_test_module()
+        ee = self._build_engine(mod, mattrs=mattrs)
+
+        arg = le.GenericValue.real(Type.double(), 1.234)
+        retval = ee.run_function(func, [arg])
+        
+        golden = math.sin(1.234)
+        answer = retval.as_real(Type.double())
+        self.assertTrue(abs(answer-golden)/golden < 1e-5)
+
+
+    def _build_engine(self, mod, mattrs):
+        if mattrs:
+            return EngineBuilder.new(mod).mattrs(mattrs).create()
+        else:
+            return EngineBuilder.new(mod).create()
+
+    def test_cpu_support2(self):
+        features = 'sse3', 'sse41', 'sse42', 'avx'
+        mattrs = ','.join(map(lambda s: '-%s' % s, features))
+        print 'disable mattrs', mattrs
+        self._template(mattrs)
+
+    def test_cpu_support3(self):
+        features = 'sse41', 'sse42', 'avx'
+        mattrs = ','.join(map(lambda s: '-%s' % s, features))
+        print 'disable mattrs', mattrs
+        self._template(mattrs)
+
+    def test_cpu_support4(self):
+        features = 'sse42', 'avx'
+        mattrs = ','.join(map(lambda s: '-%s' % s, features))
+        print 'disable mattrs', mattrs
+        self._template(mattrs)
+
+    def test_cpu_support5(self):
+        features = 'avx',
+        mattrs = ','.join(map(lambda s: '-%s' % s, features))
+        print 'disable mattrs', mattrs
+        self._template(mattrs)
+
+    def test_cpu_support6(self):
+        features = []
+        mattrs = ','.join(map(lambda s: '-%s' % s, features))
+        print 'disable mattrs', mattrs
+        self._template(mattrs)
+
+tests.append(TestCPUSupport)
+
+# ---------------------------------------------------------------------------
+class TestIntrinsicBasic(unittest.TestCase):
+
+    def _build_module(self, float):
+        mod     = Module.new('test')
+        functy  = Type.function(float, [float])
+        func    = mod.add_function(functy, "mytest%s" % float)
+        block   = func.append_basic_block("entry")
+        b       = Builder.new(block)
+        return mod, func, b
+
+    def _template(self, mod, func, pyfunc):
+        float = func.type.pointee.return_type
+        ee = le.ExecutionEngine.new(mod)
+        arg = le.GenericValue.real(float, 1.234)
+        retval = ee.run_function(func, [arg])
+        golden = pyfunc(1.234)
+        answer = retval.as_real(float)
+        self.assertTrue(abs(answer - golden) / golden < 1e-7)
+
+    def test_sqrt_f32(self):
+        float = Type.float()
+        mod, func, b = self._build_module(float)
+        intr = Function.intrinsic(mod, lc.INTR_SQRT, [float])
+        b.ret(b.call(intr, func.args))
+        self._template(mod, func, math.sqrt)
+
+    def test_sqrt_f64(self):
+        float = Type.double()
+        mod, func, b = self._build_module(float)
+        intr = Function.intrinsic(mod, lc.INTR_SQRT, [float])
+        b.ret(b.call(intr, func.args))
+        self._template(mod, func, math.sqrt)
+
+    def test_cos_f32(self):
+        if sys.platform == 'win32' and BITS == 32:
+            # float32 support is known to fail on 32-bit Windows
+            return
+        float = Type.float()
+        mod, func, b = self._build_module(float)
+        intr = Function.intrinsic(mod, lc.INTR_COS, [float])
+        b.ret(b.call(intr, func.args))
+        self._template(mod, func, math.cos)
+
+    def test_cos_f64(self):
+        float = Type.double()
+        mod, func, b = self._build_module(float)
+        intr = Function.intrinsic(mod, lc.INTR_COS, [float])
+        b.ret(b.call(intr, func.args))
+        self._template(mod, func, math.cos)
+
+    def test_sin_f32(self):
+        if sys.platform == 'win32' and BITS == 32:
+            # float32 support is known to fail on 32-bit Windows
+            return
+        float = Type.float()
+        mod, func, b = self._build_module(float)
+        intr = Function.intrinsic(mod, lc.INTR_SIN, [float])
+        b.ret(b.call(intr, func.args))
+        self._template(mod, func, math.sin)
+
+    def test_sin_f64(self):
+        float = Type.double()
+        mod, func, b = self._build_module(float)
+        intr = Function.intrinsic(mod, lc.INTR_SIN, [float])
+        b.ret(b.call(intr, func.args))
+        self._template(mod, func, math.sin)
+
+    def test_powi_f32(self):
+        float = Type.float()
+        mod, func, b = self._build_module(float)
+        intr = Function.intrinsic(mod, lc.INTR_POWI, [float])
+        b.ret(b.call(intr, [func.args[0], lc.Constant.int(Type.int(), 2)]))
+        self._template(mod, func, lambda x: x**2)
+
+    def test_powi_f64(self):
+        float = Type.double()
+        mod, func, b = self._build_module(float)
+        intr = Function.intrinsic(mod, lc.INTR_POWI, [float])
+        b.ret(b.call(intr, [func.args[0], lc.Constant.int(Type.int(), 2)]))
+        self._template(mod, func, lambda x: x**2)
+
+
+
+tests.append(TestIntrinsicBasic)
+
 # ---------------------------------------------------------------------------
 
 class TestIntrinsic(unittest.TestCase):
@@ -838,6 +1066,10 @@ class TestIntrinsic(unittest.TestCase):
         self.assertEqual(retval.as_int(), 0x42000000)
 
     def test_mysin(self):
+        if sys.platform == 'win32' and BITS == 32:
+            # float32 support is known to fail on 32-bit Windows
+            return
+
         # mysin(x) = sqrt(1.0 - pow(cos(x), 2))
         mod     = Module.new('test')
 
